package server.command;

import java.util.Collection;

import common.collect.Filter;
import server.network.Player;

public class PlayerParser extends CompletingParser {
	protected final boolean useSender;
	protected final UserPolicy policy;
	
	public PlayerParser(String name, boolean useSender, UserPolicy policy) {
		super(name);
		this.useSender = useSender;
		this.policy = policy;
	}

	public Object parse(CommandEnvironment env, String input) {
		Player net = env.getServer().getPlayer(input);
		if(net == null)
			throw new RunException("Spieler '%s' wurde nicht gefunden", input);
		if(!this.policy.applies(env, env.getExecutor(), net))
			throw new RunException(this.policy.getDenyMessage(net));
		return net;
	}
	
	public Object getDefault(CommandEnvironment env) {
		return this.useSender && env.getExecutor().isPlayer() && this.policy.applies(env, env.getExecutor(), (Player)env.getExecutor()) ? (Player)env.getExecutor() : null;
	}
	
	public Collection<String> getCompletions(CommandEnvironment env) {
		return Filter.filter(env.getServer().getAllPlayerNames(), user -> this.policy.applies(env, env.getExecutor(), env.getServer().getPlayer(user))); // add Lists.newArrayList if modifying!
	}

	public Class<?> getTypeClass(boolean required) {
		return Player.class;
	}
}
