package server.command.commands;

import java.util.Map.Entry;

import common.enchantment.Enchantment;
import common.enchantment.EnchantmentHelper;
import common.item.ItemStack;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.RunException;
import server.command.UserPolicy;

public class CommandMagic extends Command {
	public CommandMagic() {
		super("magic");
		
		this.addEnum("enchantment", Enchantment.class, Enchantment.ENCHANTMENTS);
		this.setParamsOptional();
		this.addInt("level", 1, 32767);
		this.addFlag("remove", 'r');
		this.addFlag("force", 'f');
		this.setParamsRequired();
		this.addPlayerItem("player", 'p', true, UserPolicy.NON_ADMINS_OR_SELF);
	}
	
	public void exec(CommandEnvironment env, Executor exec, Enchantment ench, Integer level, boolean remove, boolean force, ItemStack stack) {
		if(remove) {
			int current = EnchantmentHelper.getEnchantmentLevel(ench.effectId, stack);
			if(current == 0)
				throw new RunException("%s hat die Verzauberung %s nicht", stack.getDisplayName(), ench.getDisplay());
			stack.removeEnchantment(ench);
			exec.log("Verzauberung %s wurde von %s entfernt", ench.getFormattedName(current), stack.getDisplayName());
			return;
		}
		level = level == null ? ench.getMaxLevel() : level;
		if(!force) {
			if(level > ench.getMaxLevel())
				throw new RunException("Level %d ist zu hoch für %s, maximal %d erlaubt", level, ench.getDisplay(), ench.getMaxLevel());
			if(!ench.canApply(stack))
				throw new RunException("Verzauberung %s kann nicht auf %s verwendet werden", ench.getDisplay(), stack.getDisplayName());
			for(Entry<Integer, Integer> id : EnchantmentHelper.getEnchantments(stack).entrySet()) {
				Enchantment enc = Enchantment.getEnchantmentById(id.getKey());
				if(enc != ench && !ench.canApplyTogether(enc))
					throw new RunException("Verzauberung %s kann nicht zusammen mit %s verwendet werden", ench.getDisplay(), enc.getFormattedName(id.getValue()));
			}
		}
		stack.removeEnchantment(ench);
		stack.addEnchantment(ench, level);
		exec.log("Verzauberung %s wurde zu %s hinzugefügt", ench.getFormattedName(level), stack.getDisplayName());
	}
}
