package server.command.commands;

import java.util.List;

import common.entity.npc.EntityNPC;
import common.item.ItemStack;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.UserPolicy;

public class CommandRepair extends Command {
	public CommandRepair() {
		super("repair");
		
		this.setParamsOptional();
		this.addFlag("all", 'a');
		this.setParamsRequired();
		this.addPlayerEntityList("players", 'p', true, UserPolicy.NON_ADMINS_OR_SELF);
	}
	
	private boolean fixItem(ItemStack stack) {
		boolean flag = false;
		if(stack.getRepairCost() != 0) {
			stack.setRepairCost(0);
			flag = true;
		}
        if(stack.getItem().getMaxDamage() > 0 && stack.getItemDamage() != 0) {
        	stack.setItemDamage(0);
			flag = true;
		}
        return flag;
	}
	
	public void exec(CommandEnvironment env, Executor exec, boolean all, List<EntityNPC> players) {
		int done = 0;
		int repaired = 0;
		for(EntityNPC player : players) {
			int rep = 0;
			if(all) {
				for(ItemStack item : player.inventory.mainInventory) {
					if(item != null && this.fixItem(item))
						rep++;
				}
				for(ItemStack item : player.inventory.armorInventory) {
					if(item != null && this.fixItem(item))
						rep++;
				}
			}
			else if(player.getHeldItem() != null && this.fixItem(player.getHeldItem())) {
				rep++;
			}
			if(rep > 0) {
				exec.log("%d " + (rep == 1 ? "Gegenstand" : "Gegenstände") + " im Inventar von %s wurde" + (rep == 1 ? "" : "n") + " repariert", rep, player.getCommandName());
				done++;
			}
			repaired += rep;
		}
		if(done > 1)
			exec.log("%d Gegenstände im Inventar von %d Spielern wurden repariert", repaired, done);
	}
}
